@@ -1,11 +1,8 @@
 """RESQML coordinate reference systems."""
 
-version = '29th November 2021'
-
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('crs.py version ' + version)
 
 import math as maths
 import uuid
@@ -37,34 +34,33 @@ class Crs(BaseResqpy):
     valid_axis_orders = ("easting northing", "northing easting", "westing southing", "southing westing",
                          "northing westing", "westing northing")
 
-    def __init__(
-            self,
-            parent_model: 'rq.Model',
-            crs_root = None,  # deprecated
-            uuid: Optional[uuid.UUID] = None,
-            x_offset: float = 0.0,
-            y_offset: float = 0.0,
-            z_offset: float = 0.0,
-            rotation: float = 0.0,
-            xy_units: str = 'm',
-            z_units: str = 'm',
-            z_inc_down: bool = True,
-            axis_order: str = 'easting northing',
-            time_units: Optional[str] = None,
-            epsg_code: Optional[str] = None,
-            title: Optional[str] = None,
-            originator: Optional[str] = None,
-            extra_metadata: Optional[Dict[str, str]] = None):
+    def __init__(self,
+                 parent_model: 'rq.Model',
+                 uuid: Optional[uuid.UUID] = None,
+                 x_offset: float = 0.0,
+                 y_offset: float = 0.0,
+                 z_offset: float = 0.0,
+                 rotation: float = 0.0,
+                 rotation_units: str = 'dega',
+                 xy_units: str = 'm',
+                 z_units: str = 'm',
+                 z_inc_down: bool = True,
+                 axis_order: str = 'easting northing',
+                 time_units: Optional[str] = None,
+                 epsg_code: Optional[str] = None,
+                 title: Optional[str] = None,
+                 originator: Optional[str] = None,
+                 extra_metadata: Optional[Dict[str, str]] = None):
         """Create a new coordinate reference system object.
 
         arguments:
             parent_model (model.Model): the model to which the new Crs object will belong
-            crs_root (xml root node): DEPRECATED – use uuid instead; the xml root node for an existing RESQML crs object
             crs_uuid (uuid.UUID): the uuid of an existing RESQML crs object in model, from which to instantiate the
                 resqpy Crs object; if present, all the remaining arguments are ignored
             x_offset, y_offset, z_offset (floats, default zero): the local origin within an implicit parent crs
             rotation (float, default zero): a projected view rotation (in the xy plane), in radians, relative to an
-                implicit parent crs
+                implicit parent crs; positive value indicates local y axis is clockwise from global y axis.
+            rotation_units (str, default 'dega'): the units applicable to rotation, usually 'dega' or 'rad'
             xy_units (str, default 'm'): the units applicable to x & y values; must be one of the RESQML length uom strings
             z_units (str, default 'm'): the units applicable to z depth values; must be one of the RESQML length uom strings
             z_inc_down (boolean, default True): if True, increasing z values indicate greater depth (ie. in direction of
@@ -74,11 +70,11 @@ class Crs(BaseResqpy):
                 z_units argument is irrelevant
             epsg_code (str, optional): if present, the EPSG code of the implicit parent crs
             title (str, optional): the citation title to use for a new crs;
-                ignored if uuid or crs_root is not None
+                ignored if uuid is not None
             originator (str, optional): the name of the person creating the crs, defaults to login id;
-                ignored if uuid or crs_root is not None
+                ignored if uuid is not None
             extra_metadata (dict, optional): string key, value pairs to add as extra metadata for the crs;
-                ignored if uuid or crs_root is not None
+                ignored if uuid is not None
 
         returns:
             a new resqpy Crs object
@@ -103,7 +99,8 @@ class Crs(BaseResqpy):
         self.x_offset = x_offset
         self.y_offset = y_offset
         self.z_offset = z_offset
-        self.rotation = rotation  # radians
+        self.rotation = rotation
+        self.rotation_units = rotation_units
         self.axis_order = axis_order
         self.epsg_code = epsg_code
         # following are derived attributes, set below
@@ -115,31 +112,23 @@ class Crs(BaseResqpy):
                          uuid = uuid,
                          title = title,
                          originator = originator,
-                         extra_metadata = extra_metadata,
-                         root_node = crs_root)
+                         extra_metadata = extra_metadata)
 
         assert self.xy_units in wam.valid_uoms(quantity = 'length'), f'invalid CRS xy units: {self.xy_units}'
         assert self.z_units in wam.valid_uoms(quantity = 'length'), f'invalid CRS z units: {self.z_units}'
+        assert self.rotation_units in wam.valid_uoms(quantity='plane angle'), \
+            f'invalid CRS rotation units: {self.rotation_units}'
         if self.time_units:
             assert self.time_units in wam.valid_uoms(quantity = 'time'), f'invalid CRS time units: {self.time_units}'
         assert self.axis_order in self.valid_axis_orders, 'invalid CRS axis order: ' + str(axis_order)
 
-        self.rotated = (not maths.isclose(self.rotation, 0.0, abs_tol = 1e-8) and
-                        not maths.isclose(self.rotation, 2.0 * maths.pi, abs_tol = 1e-8))
-        if self.rotated:
-            self.rotation_matrix = vec.rotation_matrix_3d_axial(2, maths.degrees(-self.rotation))
-            self.reverse_rotation_matrix = vec.rotation_matrix_3d_axial(2, maths.degrees(self.rotation))
-            if self.is_right_handed_xy():
-                self.rotation_matrix, self.reverse_rotation_matrix = self.reverse_rotation_matrix, self.rotation_matrix
-
-        self.null_transform = (maths.isclose(self.x_offset, 0.0, abs_tol = 1e-8) and
-                               maths.isclose(self.y_offset, 0.0, abs_tol = 1e-8) and
-                               maths.isclose(self.z_offset, 0.0, abs_tol = 1e-8) and not self.rotated)
+        self.set_rotation_matrices()
 
     def _load_from_xml(self):
         root_node = self.root
+        assert root_node is not None
         flavour = rqet.node_type(root_node)
-        assert flavour in ['obj_LocalDepth3dCrs', 'obj_LocalTime3dCrs']
+        assert flavour in ['obj_LocalDepth3dCrs', 'obj_LocalTime3dCrs'], f'bad crs node type: {flavour}'
         if flavour == 'obj_LocalTime3dCrs':
             self.time_units = rqet.find_tag_text(root_node, 'TimeUom')
             assert self.time_units
@@ -152,22 +141,43 @@ class Crs(BaseResqpy):
         self.x_offset = rqet.find_tag_float(root_node, 'XOffset')
         self.y_offset = rqet.find_tag_float(root_node, 'YOffset')
         self.z_offset = rqet.find_tag_float(root_node, 'ZOffset')
-        self.rotation = rqet.find_tag_float(root_node, 'ArealRotation')  # todo: extract uom attribute from this node
+        self.rotation = rqet.find_tag_float(root_node, 'ArealRotation')
+        rotation_node = rqet.find_tag(root_node, 'ArealRotation')
+        self.rotation_units = rotation_node.attrib.get('uom')
         parent_xy_crs = rqet.find_tag(root_node, 'ProjectedCrs')
         if parent_xy_crs is not None and rqet.node_type(parent_xy_crs) == 'ProjectedCrsEpsgCode':
             self.epsg_code = rqet.find_tag_text(parent_xy_crs, 'EpsgCode')  # should be an integer?
         else:
             self.epsg_code = None
 
+    def set_rotation_matrices(self):
+        """Sets the rotation matrices, and the rotated and null_transform flags, call after changing rotation."""
+
+        rotation_deg = wam.convert(self.rotation, self.rotation_units, 'dega', quantity = 'plane angle')
+        # log.debug(f'setting rotation matrices for dega: {rotation_deg}')
+        self.rotated = (not maths.isclose(rotation_deg, 0.0, abs_tol = 1e-8) and
+                        not maths.isclose(rotation_deg, 360.0, abs_tol = 1e-8))
+        if self.rotated:
+            self.rotation_matrix = vec.rotation_matrix_3d_axial(2, rotation_deg)
+            self.reverse_rotation_matrix = vec.rotation_matrix_3d_axial(2, -rotation_deg)
+            if self.is_right_handed_xy():
+                self.rotation_matrix, self.reverse_rotation_matrix = self.reverse_rotation_matrix, self.rotation_matrix
+        else:
+            self.rotation_matrix = None
+            self.reverse_rotation_matrix = None
+        self.null_transform = (maths.isclose(self.x_offset, 0.0, abs_tol = 1e-8) and
+                               maths.isclose(self.y_offset, 0.0, abs_tol = 1e-8) and
+                               maths.isclose(self.z_offset, 0.0, abs_tol = 1e-8) and not self.rotated)
+
     def is_right_handed_xy(self) -> bool:
         """Returns True if the xy axes are right handed when viewed from above; False if left handed."""
 
-        return self.axis_order in ["northing easting", "southing westing", "westing northing"]
+        return bool(self.axis_order in ["northing easting", "southing westing", "westing northing"])
 
     def is_right_handed_xyz(self) -> bool:
         """Returns True if the xyz axes are right handed; False if left handed."""
 
-        return self.is_right_handed_xy() == self.z_inc_down
+        return self.is_right_handed_xy() is bool(self.z_inc_down)
 
     def global_to_local(self, xyz: PointType, global_z_inc_down: bool = True) -> Tuple[float, float, float]:
         """Convert a single xyz point from the parent coordinate reference system to this one."""
@@ -236,12 +246,12 @@ class Crs(BaseResqpy):
 
     def has_same_epsg_code(self, other_crs: 'Crs') -> bool:
         """Returns True if either of the crs'es has a null EPSG code, or if they are the same."""
-        return self.epsg_code is None or other_crs.epsg_code is None or self.epsg_code == other_crs.epsg_code
+        return not self.epsg_code or not other_crs.epsg_code or self.epsg_code == other_crs.epsg_code
 
     def is_equivalent(self, other_crs: 'Crs') -> bool:
         """Returns True if this crs is effectively the same as the other crs."""
 
-        log.debug('testing crs equivalence')
+        # log.debug('testing crs equivalence')
         if other_crs is None:
             return False
         if self is other_crs:
@@ -280,7 +290,9 @@ class Crs(BaseResqpy):
         if self is other_crs:
             return _as_xyz_tuple(xyz)
         assert self.resqml_type == other_crs.resqml_type
-        assert self.has_same_epsg_code(other_crs)
+        # assert self.has_same_epsg_code(other_crs)
+        if not self.has_same_epsg_code(other_crs):
+            log.warning("converting between crs'es with different epsg codes")
         xyz = self.local_to_global(xyz)
         # yapf: disable
         if self.resqml_type == 'LocalDepth3dCrs':
@@ -302,9 +314,11 @@ class Crs(BaseResqpy):
         """
 
         if self.is_equivalent(other_crs):
-            return
+            return xyz
         assert self.resqml_type == other_crs.resqml_type
-        assert self.has_same_epsg_code(other_crs)
+        # assert self.has_same_epsg_code(other_crs)
+        if not self.has_same_epsg_code(other_crs):
+            log.warning("converting between crs'es with different epsg codes")
         self.local_to_global_array(xyz)
         if self.resqml_type == 'LocalDepth3dCrs':
             if self.xy_units == self.z_units and other_crs.xy_units == other_crs.z_units:
@@ -327,7 +341,9 @@ class Crs(BaseResqpy):
         if self is other_crs:
             return _as_xyz_tuple(xyz)
         assert self.resqml_type == other_crs.resqml_type
-        assert self.has_same_epsg_code(other_crs)
+        # assert self.has_same_epsg_code(other_crs)
+        if not self.has_same_epsg_code(other_crs):
+            log.warning("converting between crs'es with different epsg codes")
         xyz = other_crs.local_to_global(xyz)
         # yapf: disable
         if self.resqml_type == 'LocalDepth3dCrs':
@@ -349,9 +365,11 @@ class Crs(BaseResqpy):
         """
 
         if self.is_equivalent(other_crs):
-            return
+            return xyz
         assert self.resqml_type == other_crs.resqml_type
-        assert self.has_same_epsg_code(other_crs)
+        # assert self.has_same_epsg_code(other_crs)
+        if not self.has_same_epsg_code(other_crs):
+            log.warning("converting between crs'es with different epsg codes")
         other_crs.local_to_global_array(xyz)
         if self.resqml_type == 'LocalDepth3dCrs':
             if self.xy_units == self.z_units and other_crs.xy_units == other_crs.z_units:
@@ -365,14 +383,12 @@ class Crs(BaseResqpy):
         self.global_to_local_array(xyz)
         return xyz
 
-    def create_xml(
-            self,
-            title: Optional[str] = None,
-            originator: Optional[str] = None,
-            extra_metadata: Optional[Dict[str, str]] = None,
-            add_as_part: bool = True,
-            root = None,  # deprecated
-            reuse: bool = True):
+    def create_xml(self,
+                   title: Optional[str] = None,
+                   originator: Optional[str] = None,
+                   extra_metadata: Optional[Dict[str, str]] = None,
+                   add_as_part: bool = True,
+                   reuse: bool = True):
         """Creates a Coordinate Reference System xml node and optionally adds as a part in the parent model.
 
         arguments:
@@ -382,8 +398,6 @@ class Crs(BaseResqpy):
             extra_metadata (dict, optional): string key, value pairs to add as extra metadata for the crs
             add_as_part (boolean, default True): if True the newly created crs node is added to the model
                 as a part
-            root (optional, usually None): DEPRECATED; if not None, the newly created crs node is appended
-                as a child of this node (rarely used)
             reuse (boolean, default True): if True and an equivalent crs already exists in the model then
                 the uuid for this Crs is modified to match that of the existing object and the existing
                 xml node is returned without anything new being added
@@ -421,7 +435,7 @@ class Crs(BaseResqpy):
         zoffset.text = '{0:6.4f}'.format(self.z_offset)
 
         rotation = rqet.SubElement(crs, ns['resqml2'] + 'ArealRotation')
-        rotation.set('uom', 'rad')
+        rotation.set('uom', self.rotation_units)
         rotation.set(ns['xsi'] + 'type', ns['eml'] + 'PlaneAngleMeasure')
         rotation.text = '{0:8.6f}'.format(self.rotation)
 
@@ -468,12 +482,10 @@ class Crs(BaseResqpy):
             self.model.create_unknown(root = z_crs)
         else:  # not sure if this is appropriate for the vertical crs
             z_crs.set(ns['xsi'] + 'type', ns['eml'] + 'VerticalCrsEpsgCode')
-            epsg_node = rqet.SubElement(xy_crs, ns['resqml2'] + 'EpsgCode')
+            epsg_node = rqet.SubElement(z_crs, ns['resqml2'] + 'EpsgCode')
             epsg_node.set(ns['xsi'] + 'type', ns['xsd'] + 'positiveInteger')
             epsg_node.text = str(self.epsg_code)
 
-        if root is not None:
-            root.append(crs)
         if add_as_part:
             self.model.add_part('obj_' + self.resqml_type, bu.uuid_from_string(crs.attrib['uuid']), crs)
         if self.model.crs_uuid is None:
@@ -481,12 +493,6 @@ class Crs(BaseResqpy):
 
         return crs
 
-    @property
-    def crs_root(self):
-        """DEPRECATED alias for root."""
-        warnings.warn("Attribute 'crs_root' is deprecated. Use 'root'", DeprecationWarning)
-        return self.root
-
 
 def _as_xyz_tuple(xyz):
     """Coerce into 3-tuple of floats."""
