@@ -34,7 +34,7 @@ class BaseResqpy(metaclass = ABCMeta):
         """
         raise NotImplementedError
 
-    def __init__(self, model, uuid = None, title = None, originator = None, root_node = None, extra_metadata = None):
+    def __init__(self, model, uuid = None, title = None, originator = None, extra_metadata = None):
         """Load an existing resqml object, or create new.
 
         Args:
@@ -51,23 +51,16 @@ class BaseResqpy(metaclass = ABCMeta):
             self.extra_metadata = extra_metadata
             self._standardise_extra_metadata()  # has side effect of making a copy
 
-        if root_node is not None:
-            warnings.warn("root_node parameter is deprecated, use uuid instead", DeprecationWarning)
-            uuid = rqet.uuid_for_part_root(root_node)
-
         if uuid is None:
             self.uuid = bu.new_uuid()  #: Unique identifier
-
-
-#            logger.debug(f"created new uuid for object {self}")
         else:
             self.uuid = uuid
-            #            logger.debug(f"loading existing object {self}")
-            citation_node = rqet.find_tag(self.root, 'Citation')
+            root_node = self.root
+            citation_node = rqet.find_tag(root_node, 'Citation')
             if citation_node is not None:
                 self.title = rqet.find_tag_text(citation_node, 'Title')
                 self.originator = rqet.find_tag_text(citation_node, 'Originator')
-            self.extra_metadata = rqet.load_metadata_from_xml(self.root)
+            self.extra_metadata = rqet.load_metadata_from_xml(root_node)
             self._load_from_xml()
 
     # usually overridden by derived class, unless generic code above handles all attributes
@@ -214,24 +207,3 @@ class BaseResqpy(metaclass = ABCMeta):
         for key in keys_to_display:
             html += f"<strong>{key}</strong>: {getattr(self, key)}<br>\n"
         return html
-
-    # Include some aliases for root, but raise warnings if they are used
-    # TODO: remove these aliases for self.root
-
-    @property
-    def root_node(self):
-        """DEPRECATED.
-
-        Alias for root
-        """
-        warnings.warn("Attribute 'root_node' is deprecated. Use 'root'", DeprecationWarning)
-        return self.root
-
-    @property
-    def node(self):
-        """DEPRECATED.
-
-        Alias for root
-        """
-        warnings.warn("Attribute 'node' is deprecated. Use 'root'", DeprecationWarning)
-        return self.root
