@@ -5,7 +5,6 @@ version = '24th November 2021'
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('unstructured_grid.py version ' + version)
 
 import numpy as np
 
@@ -704,7 +703,7 @@ class UnstructuredGrid(BaseResqpy):
         """
 
         face_points = self.planar_face_points(face_index, xy_plane = True)
-        local_triangulation = tri.dt(face_points)  # returns int array of shape (M, 3)
+        local_triangulation = tri.dt(face_points, algorithm = 'scipy')  # returns int array of shape (M, 3)
         assert len(
             local_triangulation) == len(face_points) - 2, 'face triangulation failure (concave edges when planar?)'
         if local_nodes:
